// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying vec3 wpos;
varying vec3 position;
varying highp vec3 p;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "util.h"
uniform float TIME;
uniform float RENDER_DISTANCE;
uniform vec2 FOG_CONTROL;
uniform vec4 FOG_COLOR;

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

vec3 acesFilm(vec3 x){
const float a = 2.65f;
const float b = 0.03f;
const float c = 2.53f;
const float d = 0.59f;
const float e = 0.14f;
return clamp((x*(a*x+b))/(x*(c*x+d)+e),0.0,1.0);}

vec3 Hyra(vec3 clr) {
float W = 1.3 / 1.0;
#ifdef enableMaps
float Luma = dot(clr, vec3(0.0, 0.6, 0.4));
vec3 Chroma = clr - Luma;
clr = (Chroma * 1.5) + Luma;
clr = vec3(filmc(clr.r), filmc(clr.g), filmc(clr.b)) / filmc(W);
#endif
return clr;
}

vec3 saturate(vec3 rgb, float adjustment){
const vec3 W = vec3(0.2125, 0.7154, 0.0721);
vec3 intensity = vec3(dot(rgb, W));
return mix(intensity, rgb, adjustment);
}

float inrect(vec2 pos, float x1, float y1, float x2, float y2, float focus){
 return min(1.0, max(min(min(pos.x - x1, x2 - pos.x), min(pos.y - y1, y2 - pos.y)), 0.0) / focus);
}

float playershadow(){
vec3 lookvector=wpos;
lookvector.x*=2.0;
 vec3 pos = lookvector + vec3(0.4, 0.4, 0.4); 
 vec3 dir = vec3(-1.0, (1.25) * 0.31, 0.0);
 float factor = 1.0;
 if (pos.x < 0.2){
  factor = max(0.0, pos.x / 0.4 + 0.5);
 }
pos += dir * pos.x;
 float focus = .04;
 float footwalk = sin((position.x - lookvector.x) * 2.0 + (position.z - lookvector.z) * 2.0);
float handswalk = sin((position.x - lookvector.x) * 2.0 + (position.z - lookvector.z)) * .5;
pos.yz -= vec2(.2, .4);
 float body = max(inrect(pos.yz, -1.5 + footwalk*.4, -0.25, 0.75, .1, focus), inrect(pos.yz, -1.5 - footwalk*.4, -.1, 0.75, 0.25, focus));
 float hands = max(inrect(pos.yz, -0.5 + footwalk * .1, -0.5, 0.25, .1, focus), inrect(pos.yz, -0.5 - footwalk * 0.1, -.1, 0.25, 0.5, focus));
 return min(1.0, max(body, hands)) * factor;
}

highp float hash(highp float p) { p = fract(p * 0.011); p *= p + 7.5; p *= p + p; return fract(p); }
highp float hash(highp vec2 p) {highp vec3 p3 = fract(vec3(p.xyx) * 0.13); p3 += dot(p3, p3.yzx + 3.333); return fract((p3.x + p3.y) * p3.z); }

highp float noise(float x) {
    highp float i = floor(x);
    highp float f = fract(x);
    highp float u = f * f * (3.0 - 2.0 * f);
    return mix(hash(i), hash(i + 1.0), u);
}

highp float noise(highp vec2 x) {
    highp vec2 i = floor(x);
    highp vec2 f = fract(x);

// Four corners in 2D of a tile
 highp float a = hash(i);
    highp float b = hash(i + vec2(1.0, 0.0));
    highp float c = hash(i + vec2(0.0, 1.0));
    highp float d = hash(i + vec2(1.0, 1.0));

    // Simple 2D lerp using smoothstep envelope between the values.
// return vec3(mix(mix(a, b, smoothstep(0.0, 1.0, f.x)),
//mix(c, d, smoothstep(0.0, 1.0, f.x)),
//smoothstep(0.0, 1.0, f.y)));

// Same code, with the clamps in smoothstep and common subexpressions
// optimized away.
    highp vec2 u = f * f * (4.6 - 3.6 * f);
return mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}

highp float noise(highp vec3 x) {
    const highp vec3 step = vec3(110, 241, 171);

    highp vec3 i = floor(x);
    highp vec3 f = fract(x);
 
    // For performance, compute the base input to a 1D hash from the integer part of the argument and the 
    // incremental change to the 1D based on the 3D -> 1D wrapping
    highp float n = dot(i, step);

    highp vec3 u = f * f * (4.6 - 3.6 * f);
    return mix(mix(mix( hash(n + dot(step, vec3(0, 0, 0))), hash(n + dot(step, vec3(1, 0, 0))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 0))), hash(n + dot(step, vec3(1, 1, 0))), u.x), u.y),
               mix(mix( hash(n + dot(step, vec3(0, 0, 1))), hash(n + dot(step, vec3(1, 0, 1))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 1))), hash(n + dot(step, vec3(1, 1, 1))), u.x), u.y), u.z);
}


void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0); 
   vec4 limits = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
   vec4 limits = texture2D(TEXTURE_1, vec2(0,1));
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

bool unwater = FOG_CONTROL.x==0.0;
float rain = smoothstep(0.66, 0.3, FOG_CONTROL.x);
bool water = (color.b>color.r);
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);

diffuse.rgb *= vec3(1.25,1,1);

if(uv1.y<0.875){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.03);}
if(uv1.y<0.873){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.02);}
if(uv1.y<0.871){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(uv1.y<0.869){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}

if(color.a != 0.0){
if(color.g<0.64){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.02);}
if(color.g<0.63){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.06);}
if(color.g<0.62){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(color.g<0.61){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}
}

highp float wave = sin(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+cos(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.
z+p.z*1.0)))+tan(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+sin(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+cos(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)));

//TimeCycle
vec4 detc = texture2D(TEXTURE_1, vec2(0.0,1.0));
float day = pow(min(max(1.1-pow(detc.r,10.0),0.0),1.0),3.0);
float night = min(pow(1.0-detc.r,2.0)*2.0,1.0);

diffuse.rgb *= vec3(1.7);
vec3 colorN = vec3(0.1,0.3,0.8);
colorN = mix(colorN, vec3(1.0), uv1.x);
diffuse.rgb *= mix(vec3(1.0),colorN,night);

float area = smoothstep( .885, .875, uv1.y);
vec3 kena = mix(vec3(0.58), vec3(1.2), uv1.x);
diffuse.rgb *= mix(vec3(1.0), kena, area);

if(unwater==false){
vec3 torch = vec3(3,0.5,0);
diffuse.rgb += diffuse.rgb* (torch* 1.3)* pow(uv1.x, 6.0)* (1.7-uv1.y);}

if(unwater== true){
vec3 sea = vec3(0.2,0.2,1.3);
vec3 source = vec3(0.0,0.0,3.0);
diffuse.rgb += sea*uv1.x;
diffuse.rgb += diffuse.rgb* (source)* pow(uv1.x, 60.0)* (1.0-uv1.y);}

#if defined(BLEND) || defined(FOG)
if(water==true){
diffuse.a = 0.786;//Opacitiy

diffuse.rgb *= vec3(0.0);//Cleared texture base

//Color in Day
vec3 clor = mix(vec3(0.22,0.65,1), vec3(1.8), clamp(length(wpos)/RENDER_DISTANCE*2.0,0.0,3.0));

//Color in Night
vec3 clorN = mix(vec3(0.19,0.3,0.6), vec3(0.5), clamp(length(wpos)/RENDER_DISTANCE*1.0,0.0,1.0));

//Color in rain
vec3 clorR = mix(vec3(0.15,0.15,0.2), vec3(4.0), clamp(length(wpos)/RENDER_DISTANCE*2.0,0.0,2.0));

//activators
vec3 lor = mix(clor,clorN, night);
diffuse.rgb += mix(lor,clorR, rain);

highp float time = TIME*1.33;
highp float wwave = noise(time+position.xz+position.zx+position.xy);
diffuse.rgb += wwave*0.098833;

diffuse.a = 0.46;
diffuse.rgb += wave*vec3(0.039,0.08,0.501)*0.01;
diffuse.rgb += noise((wpos*0.15*abs(wave)))*0.4;
diffuse.rgb += mix(vec3(0.18),vec3(0.1),abs(wave));diffuse.rgb *= vec3(1.8,1.7,1.6);
if(uv1.y<0.875){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.03);}
if(uv1.y<0.873){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.02);}
if(uv1.y<0.871){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(uv1.y<0.869){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}
if(uv1.y<0.867){diffuse.rgb *= vec3(0.78)+uv1.x*vec3(0.3);}
if(uv1.y<0.865){diffuse.rgb *= vec3(0.66)+uv1.x*vec3(0.35);}

if(color.a != 0.0){
if(color.g<0.64){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.02);}
if(color.g<0.63){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.06);}
if(color.g<0.62){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(color.g<0.61){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}
if(color.g<0.61){diffuse.rgb *=
vec3(0.78)+uv1.x*vec3(0.3);}
}
if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){

diffuse.rgb += clamp(1.0-length(-wpos.z*10.0)/RENDER_DISTANCE*1.0,0.0,1.0)*vec3(2.0,1.78,0.0)*length(-wpos.x)/RENDER_DISTANCE*2.0;}}

}
#endif

float radius = clamp(length(-wpos *0.5)/RENDER_DISTANCE *2.0,0.0,1.0);
vec3 fog = mix(diffuse.rgb, FOG_COLOR.rgb *vec3(0.96), radius);
diffuse.rgb = fog;

if(unwater==true){
diffuse.rgb *= vec3(0.4,0.5,0.63);

}

vec2 render = uv1;
render.y -= playershadow()*0.11;
diffuse *= texture2D(TEXTURE_1,render);

diffuse.rgb = acesFilm(diffuse.rgb);
diffuse.rgb = saturate(diffuse.rgb,1.5);
diffuse.rgb = Hyra(diffuse.rgb)*vec3(1.03);

if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){
	
vec3 beam = noise(vec2(wpos.z/wpos.y))*vec3(1.3,0.7,0.17);

vec3 sbeamc = mix(vec3(0.0),beam,clamp(length(wpos)/RENDER_DISTANCE*0.8,0.0,1.0));

diffuse.rgb += sbeamc;

if(unwater==false){
diffuse.rgb += vec3(-0.2)*rain;
float becek = sin(position.x+position.y+position.z)*0.045;
float deras = clamp(length(wpos.xyz)/RENDER_DISTANCE *2.0,0.0,0.4);
if(fract(position.y)==0.0){
diffuse.rgb += becek*rain*uv1.y;
diffuse.rgb += deras*rain;
}}

}}

#ifdef FOG
if(unwater==true){
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );}
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}